﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;

namespace DataSetXML
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();           
        }

        private DataTable GetCustomerTable()
        {
            DataTable dt = new DataTable("Customers");
            dt.Columns.Add("CustomerID", typeof(Int32));
            dt.Columns.Add("CustomerName", typeof(String));
            dt.Columns.Add("Description", typeof(string));
            dt.Columns.Add("Boolean", typeof(Boolean));
            DataRow row = dt.NewRow();
            row[0] = 001;
            row[1] = "大众";
            row[2] = "中档车";
            row[3] = true;
            dt.Rows.Add(row);

            DataRow row2 = dt.NewRow();
            row2[0] = 002;
            row2[1] = "奔驰";
            row2[2] = "豪车";
            row2[3] = false;
            dt.Rows.Add(row2);

           
            return dt;
        }
        private DataSet compareDataSet = new DataSet("CompareTime");

        private string compareXmlName = "CompareXml";
        private string compareXmlSchemaName = "CompareXmlSchema";
        private void button7_Click(object sender, EventArgs e)
        {
            DataSet compareDataSet1 = new DataSet("CompareDataSet");
            Stopwatch sw = Stopwatch.StartNew();
            //读取架构信息
            compareDataSet1.ReadXmlSchema(compareXmlSchemaName);
            //防止在修改数据的过程中引发各种约束的检查,并可以提高速度
            foreach (DataTable dt in compareDataSet.Tables)
            {
                dt.BeginLoadData();
            }
            compareDataSet1.ReadXml(compareXmlName, XmlReadMode.IgnoreSchema);
            foreach (DataTable dt in compareDataSet1.Tables)
            {
                dt.EndLoadData();
            }
            TimeSpan ts = sw.Elapsed;
            this.dataGridView3.DataSource = compareDataSet1.Tables[0];
            textBox6.Text = ts.TotalMilliseconds.ToString();
        }

        private void button8_Click(object sender, EventArgs e)
        {
            DataSet compareDataSet1 = new DataSet("CompareDataSet");
            Stopwatch sw = Stopwatch.StartNew();
            //读取架构信息

            compareDataSet1.ReadXml(compareXmlName, XmlReadMode.Auto);
           
            TimeSpan ts = sw.Elapsed;
            this.dataGridView3.DataSource = compareDataSet1.Tables[0];
            textBox7.Text = ts.TotalMilliseconds.ToString();
        }

        private void button9_Click(object sender, EventArgs e)
        {
            DataSet compareDataSet1 = new DataSet("CompareDataSet");
            Stopwatch sw = Stopwatch.StartNew();
            //读取架构信息
            compareDataSet1.ReadXmlSchema(compareXmlSchemaName);
           
            compareDataSet1.ReadXml(compareXmlName, XmlReadMode.IgnoreSchema);
          
            TimeSpan ts = sw.Elapsed;
            this.dataGridView3.DataSource = compareDataSet1.Tables[0];
            textBox8.Text = ts.TotalMilliseconds.ToString();
        }

        private void button10_Click(object sender, EventArgs e)
        {
            Stopwatch sw = Stopwatch.StartNew();
            List<Dictionary<string, string>> entityInfo = new List<Dictionary<string, string>>();
            using (XmlReader reader = new XmlTextReader(compareXmlName))
            {
                Dictionary<string, string> xmlValue = null;
                string key = string.Empty;
                while (reader.Read())
                {
                    switch (reader.NodeType)
                    {
                        case XmlNodeType.Element:
                            if (string.Compare(reader.LocalName, "MultiTable",
                                StringComparison.OrdinalIgnoreCase) == 0)
                            {
                                xmlValue = new Dictionary<string, string>();
                            }
                            else if(string.Compare(reader.LocalName,"NewDataSet",
                                StringComparison.OrdinalIgnoreCase) != 0)
                            {
                                key = reader.LocalName;
                            }
                            break;
                        case XmlNodeType.EndElement:
                            if (string.Compare(reader.LocalName, "MultiTable",
                                StringComparison.OrdinalIgnoreCase) == 0)
                            {
                                if (xmlValue != null)
                                {
                                    entityInfo.Add(xmlValue);
                                    xmlValue = null;
                                }
                            }                      
                            break;
                        case XmlNodeType.Text:
                            if (xmlValue != null)
                            {
                                xmlValue.Add(key, reader.Value);
                            }
                            break;
                    }
                }
            }
            TimeSpan ts = sw.Elapsed;

            List<Customers> packData = new List<Customers>();
            foreach (var item in entityInfo)
            {
                Customers temp = new Customers();
                foreach (var sub in item)
                {
                    if (sub.Key == "CustomerID")
                    {
                        temp.CustomerID = int.Parse(sub.Value);
                    }
                    else if (sub.Key == "CustomerName")
                    {
                        temp.CustomerName = sub.Value;
                    }
                    else if (sub.Key == "Description")
                    {
                        temp.Description = sub.Value;
                    }
                    else if (sub.Key == "Boolean")
                    {
                        temp.IsEnabled = sub.Value == "true" ? true : false;
                    }
                }
                packData.Add(temp);
            }

            dataGridView3.DataSource = packData;
            this.textBox9.Text = ts.TotalMilliseconds.ToString();
        }

        class Customers
        {
            private bool isEnabled;

            public bool IsEnabled
            {
                get { return isEnabled; }
                set { isEnabled = value; }
            }
            private int customerID;

            public int CustomerID
            {
                get { return customerID; }
                set { customerID = value; }
            }
            private string customerName;

            public string CustomerName
            {
                get { return customerName; }
                set { customerName = value; }
            }

            private string description;

            public string Description
            {
                get { return description; }
                set { description = value; }
            }
            
        }

        private void button11_Click(object sender, EventArgs e)
        {
            Stopwatch sw = Stopwatch.StartNew();
            XmlDocument doc = new XmlDocument();
            doc.Load(compareXmlName);
            XmlNode root = doc.DocumentElement;
            XmlNode child = root.FirstChild;
            List<Customers> results = new List<Customers>();
            while (child != null)
            {
                XmlNode grandSon = child.FirstChild;
                Customers temp = new Customers();
                while (grandSon != null)
                {
                    if (grandSon.Name == "CustomerID")
                    {
                        temp.CustomerID = int.Parse(grandSon.FirstChild.Value);
                    }
                    else if (grandSon.Name == "CustomerName")
                    {
                        temp.CustomerName = grandSon.FirstChild.Value;
                    }
                    else if (grandSon.Name == "Description")
                    {
                        temp.Description = grandSon.FirstChild.Value;
                    }
                    else if (grandSon.Name == "Boolean")
                    {
                        temp.IsEnabled = grandSon.FirstChild.Value == "true" ? true : false;
                    }
                    grandSon = grandSon.NextSibling;
                }
                results.Add(temp);
                child = child.NextSibling;
            }
            TimeSpan ts = sw.Elapsed;

            dataGridView3.DataSource = results;
            textBox10.Text = ts.TotalMilliseconds.ToString();
        }

        private string writeXmlName = "WriteXmlName";
        private string writeXmlNameSchema = "WriteXmlNameSchema";
        private void button1_Click(object sender, EventArgs e)
        {
            DataSet sets = new DataSet();
            sets.Tables.Add(ConstructDataTable(int.Parse(textBox1.Text)));
            sets.WriteXml(writeXmlName);
            sets.WriteXmlSchema(writeXmlNameSchema);
            textBox2.Text = sets.GetXml();
            dataGridView1.DataSource = sets.Tables[0];
        }

        private DataTable ConstructDataTable(int number)
        {
            DataTable dt = new DataTable("Customers");
            dt.Columns.Add("CustomerID", typeof(Int32));
            dt.Columns.Add("CustomerName", typeof(String));
            dt.Columns.Add("Description", typeof(string));
            dt.Columns.Add("Boolean", typeof(Boolean));

            for (int i = 0; i < number; i++)
            {
                DataRow row = dt.NewRow();
                row[0] = i;
                row[1] = "大众";
                row[2] = "中档车";
                row[3] = i / 2 == 0 ? true : false;
                dt.Rows.Add(row);
            }
            return dt;
           
        }

       
    }
}
